==> ikapi-2.1.2.tgz: package/src/Database/DDB/IKDBQueryUpdate.js <==
import IKDBBaseQuery from './IKDBBaseQuery';
import Utils from './../../API/IKUtils';
//
export default class IKDBQueryUpdate extends IKDBBaseQuery {
  constructor(indexName) {
    super(indexName)
    this.setValues = {};
    this.removeValues = [];
    this.incrementValues = {};
    this.decrementValues = {};
  }
  /*public methods*/
  appendDefaultTrail(userID) {
    this.appendValue(userID, 'updatedBy');
    this.appendValue(Date.now(), 'updatedOn');
  }
  //return if the value was appended or not
  appendValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
       ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.setValues[key] = val;
      console.debug(`Append ${key}@${val}`);
      return true;
    } return false;
  }
  //return if the value was appended or not
  incrementValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
      ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.incrementValues[key] = val;
      console.debug(`Increment ${key}@${val}`);
      return true;
    } return false;
  }
  //return if the value was appended or not
  decrementValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
      ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.decrementValues[key] = val;
      console.debug(`Decrement ${key}@${val}`);
      return true;
    } return false;
  }
  //dbKey and prefix are optional values
  appendObjectForKeyValue(source, key, dbKey, prefix) {
    //Check for optional db key that can differ from input key
    if (dbKey == undefined || dbKey == null) dbKey = key;
    //Check if valid, if yes, add, if not, remove
    if (source[key] != undefined &&
       ((Object.keys(source[key]).length > 0) ||
        (!isNaN(source[key]) && source[key].length != 0) ||
        (source[key].length > 0))) {
          //Check for object conversion
          let val = (prefix != undefined && source[key].indexOf(prefix) == -1 ?
                     prefix + source[key] : source[key]);
          //valid
          if (val != undefined) { this.setValues[dbKey]= val; }
          console.debug(`Append-ofk ${dbKey}@${val}`);
      //remove
    } else this.removeValues.push(dbKey);
  }
  //
  async run(dbManager) {
    const resp = await this.rawRun(dbManager);
    if (resp.Attributes) {
      const unmarshalled = Utils.decapsulateForDB((Array.isArray(resp.Attributes) ? resp.Attributes : [resp.Attributes]));
      console.debug('Parsed result: ', unmarshalled);
      return unmarshalled;
    } return false;
  }
  async rawRun(dbManager) {
    let query = this._rawQuery(dbManager.tableName);
    console.log("Updating item: ", query);
    let resp = await dbManager.connection.updateItem(query).promise();
    console.debug('Raw result: ', resp);
    return resp;
  }

  /*private methods*/
  _rawQuery(dbTableName, isTransaction) {
    return this._buildUpdateQuery(
            { ...super._rawQuery(dbTableName, isTransaction),
              ...(isTransaction ? {} : {ReturnValues: "ALL_OLD" /*request all updated values*/ })});
  }
  //Builds UpdateExpression
  _buildUpdateQuery(expression) {
    console.debug('Building update query..');
    //Update ops
    expression = this.__buildUpdateItems(expression);
    //Increment ops
    expression = this.__buildIncrementItems(expression);
    //Decrement ops
    expression = this.__buildDecrementItems(expression);
    //REMOVE ops
    expression = this.__buildRemoveItems(expression);
    return expression;
  }
  __buildUpdateItems(expression) {
    if (Object.keys(this.setValues).length > 0) {
      //check required values
      if (!expression.UpdateExpression) expression.UpdateExpression = " SET ";
      else expression.UpdateExpression += " SET ";
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      //build update expression
      for (let dbKey of Object.keys(this.setValues)) {
        let appendingKey = (':' + dbKey);
        let appendingValueKey = ('#' + dbKey );
        let val = this.setValues[dbKey];
        if (val != undefined) {
          let last = Object.keys(this.setValues).indexOf(dbKey) == (Object.keys(this.setValues).length - 1);
          //last or first
          if (last) expression.UpdateExpression += appendingValueKey + ' =' + appendingKey;
          else expression.UpdateExpression += appendingValueKey + ' =' + appendingKey + ', ';
          //
          expression.ExpressionAttributeValues[appendingKey] = Utils.encapsulateForDB(val);
          expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
        }
      }
    } return expression;
  }
  __buildIncrementItems(expression) {
    if (Object.keys(this.incrementValues).length > 0) {
      //check required values
      if (!expression.UpdateExpression) expression.UpdateExpression = " SET ";
      else if (expression.UpdateExpression.indexOf(' SET ') == -1) expression.UpdateExpression += " SET ";
      else expression.UpdateExpression += ', ';
      //
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      //build update expression
      for (let dbKey of Object.keys(this.incrementValues)) {
        let appendingKey = (':' + dbKey);
        let appendingValueKey = ('#' + dbKey);
        let val = this.incrementValues[dbKey];
        if (val != undefined) {
          let last = Object.keys(this.incrementValues).indexOf(dbKey) == (Object.keys(this.incrementValues).length - 1);
          //last
          if (last) expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '+' + appendingKey;
          else {
            let appendExists = (Object.keys(this.setValues).length > 0);
            //if is first and append values exists, add comman beforeß
            if (Object.keys(this.incrementValues).indexOf(dbKey) == 0 && appendExists) expression.UpdateExpression += ', ' + appendingValueKey + ' =' + appendingValueKey + '+' + appendingKey + ', ';
            else expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '+' + appendingKey + ', ';
          }
          //
          expression.ExpressionAttributeValues[appendingKey] = Utils.encapsulateForDB(val);
          expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
        }
      }
    } return expression;
  }
  __buildDecrementItems(expression) {
    if (Object.keys(this.decrementValues).length > 0) {
      //check required values
      if (!expression.UpdateExpression) expression.UpdateExpression = " SET ";
      else if (expression.UpdateExpression.indexOf(' SET ') == -1) expression.UpdateExpression += " SET ";
      else expression.UpdateExpression += ', ';
      //
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      //build update expression
      for (let dbKey of Object.keys(this.decrementValues)) {
        let appendingKey = (':' + dbKey);
        let appendingValueKey = ('#' + dbKey);
        let val = this.decrementValues[dbKey];
        if (val != undefined) {
          let last = Object.keys(this.decrementValues).indexOf(dbKey) == (Object.keys(this.decrementValues).length - 1);
          //last
          if (last) expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '-' + appendingKey;
          else {
            let appendExists = (Object.keys(this.setValues).length > 0);
            let incrementExists = (Object.keys(this.incrementValues).length > 0);
            //if is first and append values exists, add comman before
            if (Object.keys(this.decrementValues).indexOf(dbKey) == 0 && (appendExists || incrementExists)) expression.UpdateExpression += ', ' + appendingValueKey + ' =' + appendingValueKey + '-' + appendingKey + ', ';
            else expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '-' + appendingKey + ', ';
          }
          //
          expression.ExpressionAttributeValues[appendingKey] = Utils.encapsulateForDB(val);
          expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
        }
      }
    } return expression;
  }
  __buildRemoveItems(expression) {
    if (this.removeValues != undefined && this.removeValues.length > 0) {
      //Check for requirements
      if (!expression.UpdateExpression) expression.UpdateExpression = " REMOVE ";
      else expression.UpdateExpression += " REMOVE ";
      //build delete expression
      for (let dbKey of this.removeValues) {
        let appendingValueKey = ('#' + dbKey );
        let last = this.removeValues.indexOf(dbKey) == (this.removeValues.length - 1);
        //first or last
        if (last) expression.UpdateExpression += appendingValueKey;
        else expression.UpdateExpression += appendingValueKey + ', ';
        //
        expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
      }
    } return expression;
  }
}

==> ikapi-2.1.7.tgz: package/src/Database/DDB/IKDBQueryUpdate.js <==
import IKDBBaseQuery from './IKDBBaseQuery';
import Utils from './../../API/IKUtils';
//
export default class IKDBQueryUpdate extends IKDBBaseQuery {
  constructor(indexName) {
    super(indexName)
    this.setValues = {};
    this.removeValues = [];
    this.incrementValues = {};
    this.decrementValues = {};
  }
  /*public methods*/
  appendDefaultTrail(userID) {
    this.appendValue(userID, 'updatedBy');
    this.appendValue(Date.now(), 'updatedOn');
  }
  //return if the value was appended or not
  appendValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
       ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.setValues[key] = val;
      console.debug(`Append ${key}@${val}`);
      return true;
    } return false;
  }
  //return if the value was appended or not
  incrementValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
      ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.incrementValues[key] = val;
      console.debug(`Increment ${key}@${val}`);
      return true;
    } return false;
  }
  //return if the value was appended or not
  decrementValue(val, key, prefix) {
    //check if value is valid
    if (val != undefined &&
      ((Object.keys(val).length > 0) ||
        (!isNaN(val) && val.length != 0) ||
        (val.length > 0))) {
      //Check for object conversion
      val = (prefix != undefined && val.indexOf(prefix) == -1 ? prefix + val : val);
      //valid
      this.decrementValues[key] = val;
      console.debug(`Decrement ${key}@${val}`);
      return true;
    } return false;
  }
  //dbKey and prefix are optional values
  appendObjectForKeyValue(source, key, dbKey, prefix) {
    //Check for optional db key that can differ from input key
    if (dbKey == undefined || dbKey == null) dbKey = key;
    //Check if valid, if yes, add, if not, remove
    if (source[key] != undefined &&
       ((Object.keys(source[key]).length > 0) ||
        (!isNaN(source[key]) && source[key].length != 0) ||
        (source[key].length > 0))) {
          //Check for object conversion
          let val = (prefix != undefined && source[key].indexOf(prefix) == -1 ?
                     prefix + source[key] : source[key]);
          //valid
          if (val != undefined) { this.setValues[dbKey]= val; }
          console.debug(`Append-ofk ${dbKey}@${val}`);
      //remove
    } else this.removeValues.push(dbKey);
  }
  //
  async run(dbManager) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const resp = await this.rawRun(dbManager);
    if (resp.Attributes) {
      const unmarshalled = Utils.decapsulateForDB((Array.isArray(resp.Attributes) ? resp.Attributes : [resp.Attributes]));
      localConsole.debug('Parsed result: ', unmarshalled);
      return unmarshalled;
    } return false;
  }
  async rawRun(dbManager) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    let query = this._rawQuery(dbManager.tableName);
    localConsole.log("Updating item: ", query);
    let resp = await dbManager.connection.updateItem(query).promise();
    localConsole.debug('Raw result: ', resp);
    return resp;
  }

  /*private methods*/
  _rawQuery(dbTableName, isTransaction) {
    return this._buildUpdateQuery(
            { ...super._rawQuery(dbTableName, isTransaction),
              ...(isTransaction ? {} : {ReturnValues: "ALL_OLD" /*request all updated values*/ })});
  }
  //Builds UpdateExpression
  _buildUpdateQuery(expression) {
    console.debug('Building update query..');
    //Update ops
    expression = this.__buildUpdateItems(expression);
    //Increment ops
    expression = this.__buildIncrementItems(expression);
    //Decrement ops
    expression = this.__buildDecrementItems(expression);
    //REMOVE ops
    expression = this.__buildRemoveItems(expression);
    return expression;
  }
  __buildUpdateItems(expression) {
    if (Object.keys(this.setValues).length > 0) {
      //check required values
      if (!expression.UpdateExpression) expression.UpdateExpression = " SET ";
      else expression.UpdateExpression += " SET ";
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      //build update expression
      for (let dbKey of Object.keys(this.setValues)) {
        let appendingKey = (':' + dbKey);
        let appendingValueKey = ('#' + dbKey );
        let val = this.setValues[dbKey];
        if (val != undefined) {
          let last = Object.keys(this.setValues).indexOf(dbKey) == (Object.keys(this.setValues).length - 1);
          //last or first
          if (last) expression.UpdateExpression += appendingValueKey + ' =' + appendingKey;
          else expression.UpdateExpression += appendingValueKey + ' =' + appendingKey + ', ';
          //
          expression.ExpressionAttributeValues[appendingKey] = Utils.encapsulateForDB(val);
          expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
        }
      }
    } return expression;
  }
  __buildIncrementItems(expression) {
    if (Object.keys(this.incrementValues).length > 0) {
      //check required values
      if (!expression.UpdateExpression) expression.UpdateExpression = " SET ";
      else if (expression.UpdateExpression.indexOf(' SET ') == -1) expression.UpdateExpression += " SET ";
      else expression.UpdateExpression += ', ';
      //
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      //build update expression
      for (let dbKey of Object.keys(this.incrementValues)) {
        let appendingKey = (':' + dbKey);
        let appendingValueKey = ('#' + dbKey);
        let val = this.incrementValues[dbKey];
        if (val != undefined) {
          let last = Object.keys(this.incrementValues).indexOf(dbKey) == (Object.keys(this.incrementValues).length - 1);
          //last
          if (last) expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '+' + appendingKey;
          else {
            let appendExists = (Object.keys(this.setValues).length > 0);
            //if is first and append values exists, add comman beforeß
            if (Object.keys(this.incrementValues).indexOf(dbKey) == 0 && appendExists) expression.UpdateExpression += ', ' + appendingValueKey + ' =' + appendingValueKey + '+' + appendingKey + ', ';
            else expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '+' + appendingKey + ', ';
          }
          //
          expression.ExpressionAttributeValues[appendingKey] = Utils.encapsulateForDB(val);
          expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
        }
      }
    } return expression;
  }
  __buildDecrementItems(expression) {
    if (Object.keys(this.decrementValues).length > 0) {
      //check required values
      if (!expression.UpdateExpression) expression.UpdateExpression = " SET ";
      else if (expression.UpdateExpression.indexOf(' SET ') == -1) expression.UpdateExpression += " SET ";
      else expression.UpdateExpression += ', ';
      //
      if (!expression.ExpressionAttributeValues) expression.ExpressionAttributeValues = {};
      if (!expression.ExpressionAttributeNames) expression.ExpressionAttributeNames = {};
      //build update expression
      for (let dbKey of Object.keys(this.decrementValues)) {
        let appendingKey = (':' + dbKey);
        let appendingValueKey = ('#' + dbKey);
        let val = this.decrementValues[dbKey];
        if (val != undefined) {
          let last = Object.keys(this.decrementValues).indexOf(dbKey) == (Object.keys(this.decrementValues).length - 1);
          //last
          if (last) expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '-' + appendingKey;
          else {
            let appendExists = (Object.keys(this.setValues).length > 0);
            let incrementExists = (Object.keys(this.incrementValues).length > 0);
            //if is first and append values exists, add comman before
            if (Object.keys(this.decrementValues).indexOf(dbKey) == 0 && (appendExists || incrementExists)) expression.UpdateExpression += ', ' + appendingValueKey + ' =' + appendingValueKey + '-' + appendingKey + ', ';
            else expression.UpdateExpression += appendingValueKey + ' =' + appendingValueKey + '-' + appendingKey + ', ';
          }
          //
          expression.ExpressionAttributeValues[appendingKey] = Utils.encapsulateForDB(val);
          expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
        }
      }
    } return expression;
  }
  __buildRemoveItems(expression) {
    if (this.removeValues != undefined && this.removeValues.length > 0) {
      //Check for requirements
      if (!expression.UpdateExpression) expression.UpdateExpression = " REMOVE ";
      else expression.UpdateExpression += " REMOVE ";
      //build delete expression
      for (let dbKey of this.removeValues) {
        let appendingValueKey = ('#' + dbKey );
        let last = this.removeValues.indexOf(dbKey) == (this.removeValues.length - 1);
        //first or last
        if (last) expression.UpdateExpression += appendingValueKey;
        else expression.UpdateExpression += appendingValueKey + ', ';
        //
        expression.ExpressionAttributeNames[appendingValueKey] = dbKey;
      }
    } return expression;
  }
}
